"""
Soft Potato pip version

"""

from .calculate import *
from .simulate import *
from .plotting import *
from .technique import *

__version__ = "1.0.14"
__author__ = 'Oliver Rodriguez'

__all__ = ['technique', 'calculate', 'plotting', 'simulate']
